import path from 'path'
import webpack from "webpack";

import FriendlyErrorsPlugin from "friendly-errors-webpack-plugin";
import OptimizeCssAssetsPlugin from "optimize-css-assets-webpack-plugin";
import MiniCssExtractPlugin from "mini-css-extract-plugin";
import { VueLoaderPlugin } from "vue-loader";
import { BundleAnalyzerPlugin } from 'webpack-bundle-analyzer'

import options from "./config/options";

import {searchDirs, searchEntries} from "./utils";

const baseDir = path.resolve('../plugins')
const allPluginDirs = searchDirs(baseDir, 'vue3')

const generateConfig = (pluginDir) => {
  const pluginName = pluginDir.split('/').pop().toLowerCase()
  const entry = searchEntries({
    fileName: "index",
    entryPath: pluginDir + '/vue3/js',
    fileNamePrefix: `${pluginName}/vue3/js/`,
    isFuzzy: true,
  })

  const config = {
    watch: options.__DEV__,
    watchOptions: {
      ignored: /node_modules/,
    },
    mode: process.env.NODE_ENV,
    entry,
    output: Object.assign(options.output, {
      filename: "[name].js",
    }),
    externals: options.externals,
    resolve: {
      alias: {
        app: path.resolve('../', 'app/Resources/static-src/app'),
        common: path.resolve('../', 'app/Resources/static-src/common'),
        vue3: path.resolve('../', 'vue3'),
        'vue$': 'vue/dist/vue.esm-bundler.js',
        'vue-i18n$': 'vue-i18n/dist/vue-i18n.esm-bundler.js',
        pluginName: pluginDir,
      },
      extensions: [".js", ".jsx", ".vue", ".ts", ".tsx"],
    },
    module: {
      rules: [
        {
          test: /\.vue$/,
          loader: "vue-loader",
        },
        {
          test: /\.js[x]?$/,
          use: [
            {
              loader: "thread-loader",
              options: {
                workers: options.cpuNumber,
              },
            },
            {
              loader: "babel-loader",
            },
          ],
          exclude: [path.resolve('./', 'node_modules')],
        },
        {
          test: /\.ts$/,
          loader: 'ts-loader',
          exclude: /node_modules/, // 不编译node_modules下的文件
          options: { appendTsSuffixTo: [/\.vue$/] }
        },
        {
          test: /\.css$/,
          use: [
            options.__DEV__ ? 'vue-style-loader' : MiniCssExtractPlugin.loader,
            { loader: 'css-loader', options: { esModule: false }},
            'postcss-loader'
          ]
        },
        {
          test: /\.less$/,
          use: [
            options.__DEV__ ? 'vue-style-loader' : MiniCssExtractPlugin.loader,
            'css-loader',
            'postcss-loader',
            'less-loader'
          ]
        },
        {
          test: /\.(png|jpe?g|gif)$/,
          use: [
            {
              loader: `file-loader`,
              options: {
                outputPath: (url, resourcePath) => {
                  return pluginName + '/vue3' + resourcePath.split('vue3')[1]
                }
              },
            }
          ]
        },
        {
          test: /\.svg$/i,
          exclude: [path.resolve('./', 'node_modules')],
          use: [
            {
              loader: 'url-loader',
              options: {
                encoding: "base64",
              },
            },
          ]
        },
      ],
    },
    optimization: {
      splitChunks: {
        cacheGroups: {
          common: {
            name: `${pluginName}/vue3/js/${options.commonsChunkFileName}`,
            chunks: "initial", //入口处开始提取代码
            minSize: 100 * 1024, //代码最小多大，进行抽离
            minChunks: 2,
          },
        },
      },
    },
    plugins: [
      new MiniCssExtractPlugin(),
      new webpack.DefinePlugin({
        // __webpack_public_path__: `window.__publicPath`,
        "process.env": {
          NODE_ENV: JSON.stringify(process.env.NODE_ENV || "development"),
        },
      }),
      // new webpack.ProvidePlugin(options.global),
      new webpack.ContextReplacementPlugin(
        /moment[\\\/]locale$/,
        /^\.\/(zh-cn|en-gb)+\.js$/
      ),
      new VueLoaderPlugin(),
    ],
  };

  if (options.__ANALYZER__) {
    config.plugins = config.plugins.concat(new BundleAnalyzerPlugin());
  }

  config.plugins = config.plugins.concat(new FriendlyErrorsPlugin());

  if (process.env.NODE_ENV === 'production') {
    config.plugins = config.plugins.concat(new OptimizeCssAssetsPlugin());
  } else {
    config.devtool = 'source-map';
  }

  return config
}

let watchPluginDirs = allPluginDirs.filter(pluginDir => {
  if (options.ignoreModule.indexOf('plugin') > -1) {
    return false
  }

  const pluginName = pluginDir.split('/').pop().toLowerCase()
  
  if (options.ignoreModule.indexOf(pluginName) > -1) {
    return true
  }

  return false
})

watchPluginDirs = allPluginDirs.filter(pluginDir => {
  const pluginName = pluginDir.split('/').pop().toLowerCase()

  if (options.watchModule.indexOf(pluginName) > -1) {
    return false
  }

  return true
})

const configs = []
watchPluginDirs.forEach(pluginDir => {
  if (options.ignoreModule.indexOf('plugin') > -1 && options.watchModule.length === 0) {
    return
  }

  const pluginName = pluginDir.split('/').pop().toLowerCase()

  if (options.ignoreModule.indexOf(pluginName) > -1) {
    return
  }

  configs.push(generateConfig(pluginDir))
})

export default configs;
